#!/usr/bin/env python

import sys
import django
from django.conf import settings
from django.test.utils import get_runner


APP_NAME = 'django_forum_app'

settings.configure(
    DEBUG=True,
    DATABASES={
        'default': {
            'ENGINE': 'django.db.backends.mysql',
        }
    },
    USE_TZ=True,
    ROOT_URLCONF='{0}.tests'.format(APP_NAME),
    SITE_ID=1,
    INSTALLED_APPS=(
        'django.contrib.auth',
        'django.contrib.contenttypes',
        'django.contrib.sessions',
        'django.contrib.admin',
        'django.contrib.sites',
        'photologue',
        'tinymce',
        APP_NAME,
    ),
    TEMPLATE_CONTEXT_PROCESSORS=(
        "django.contrib.auth.context_processors.auth",
        "django.core.context_processors.debug",
        "django.core.context_processors.i18n",
        "django.core.context_processors.media",
        "django.core.context_processors.request",
    ),
)

if hasattr(django, 'setup'):
    django.setup()
TestRunner = get_runner(settings)
test_runner = TestRunner()
failures = test_runner.run_tests([APP_NAME])
if failures:
    sys.exit(failures)



from django.conf.urls import url
from django_forum_app import views

urlpatterns = [
    url(r'^$', views.index, name='forum-index'),
    url(r'^(?P<slug>[-\w]+)/$', views.forum, name='forum-detail'),
    url(r'^(?P<slug>[-\w]+)/(?P<topic_id>\d+)/$', views.topic, name='topic-detail'),
    url(r'^(?P<slug>[-\w]+)/(?P<topic_id>\d+)/close$', views.close_topic, name='close-topic'),
    url(r'^(?P<slug>[-\w]+)/(?P<topic_id>\d+)/reply/$', views.post_reply, name='reply'),
    url(r'(?P<slug>[-\w]+)/newtopic/$', views.new_topic, name='new-topic'),
]



from django.db import models
from django.conf import settings
from django.db.models import Count
from django.core.mail import send_mail
from django.db.models.signals import post_save
from photologue.models import Photo
from django.utils.translation import ugettext_lazy as _
try:
    User = settings.AUTH_USER_MODEL
except ImportError:  # django < 1.5
    from django.contrib.auth.models import User


class Category(models.Model):
    order = models.IntegerField()
    title = models.CharField(max_length=60)

    def get_forums(self):
        return Forum.objects.filter(category=self.id)

    def __unicode__(self):
        return self.title


class Forum(models.Model):
    title = models.CharField(max_length=60)
    slug = slug = models.SlugField(unique=True)
    description = models.TextField(blank=True, default='')
    icon = models.ForeignKey(Photo, null=True, blank=True)
    category = models.ForeignKey(Category)
    updated = models.DateTimeField(auto_now=True)
    created = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(User, blank=True, null=True)

    def __unicode__(self):
        return self.title

    def get_summary(self):
        if len(self.description) > 50:
            return self.description[:50] + '...'
        return self.description

    def get_visits(self):
        vs = 0
        for t in self.topic_set.all():
            vs += t.visits
        return vs

    def has_seen(self, user=None):
        if user.is_authenticated():
            for t in self.topic_set.all():
                if not t.has_seen(user):
                    return False
        return True

    def num_posts(self):
        return sum([t.num_posts() for t in self.topic_set.all()])

    def num_topics(self):
        return self.topic_set.all().count()

    def last_post(self):
        if self.topic_set.count():
            last = None
            for t in self.topic_set.all():
                l = t.last_post()
                if l:
                    if not last:
                        last = l
                    elif l.created > last.created:
                        last = l
            return last


class Topic(models.Model):
    title = models.CharField(max_length=60)
    description = models.TextField(max_length=10000, blank=True, null=True)
    # forum = models.ForeignKey(Forum)
    forums = models.ManyToManyField(Forum)
    block_top = models.BooleanField(default=False)
    created = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(User, blank=True, null=True)
    updated = models.DateTimeField(auto_now=True)
    closed = models.BooleanField(blank=True, default=False)
    visits = models.IntegerField(default=0)
    user_lst = models.TextField(blank=True, null=True)

    def num_posts(self):
        return self.post_set.count()

    def num_replies(self):
        return max(0, self.post_set.count() - 1)

    def last_post(self):
        if self.post_set.count():
            return self.post_set.order_by("-created")[0]

    def sum_visits(self, user_id=None):
        if user_id:
            if self.user_lst:
                lst = self.user_lst.split(',')
                if str(user_id) not in lst:
                    self.user_lst += ',' + str(user_id)
            else:
                self.user_lst = str(user_id)
        self.visits += 1
        self.save()

    def has_seen(self, user=None):
        if user.is_authenticated():
            if self.user_lst:
                lst = self.user_lst.split(',')
                if str(user.id) in lst:
                    return True
            return False
        return True

    def __unicode__(self):
        return unicode(self.creator) + " - " + self.title


class Post(models.Model):
    title = models.CharField(max_length=60, verbose_name="Izenburua")
    created = models.DateTimeField(auto_now_add=True)
    creator = models.ForeignKey(User, blank=True, null=True, related_name="%(class)s_posts")
    updated = models.DateTimeField(auto_now=True)
    topic = models.ForeignKey(Topic)
    body = models.TextField(max_length=10000)
    user_ip = models.GenericIPAddressField(blank=True, null=True)
    telegram_id = models.CharField(max_length=20, blank=True, null=True)

    def __unicode__(self):
        return u"%s - %s - %s" % (self.creator, self.topic, self.title)

    def get_post_num(self):
        return Post.objects.filter(topic__id=self.topic_id).filter(created__lt=self.created).count()

    def get_page(self):
        return self.get_post_num() / settings.POSTS_PER_PAGE + 1

    def short(self):
        return u"%s - %s\n%s" % (self.creator, self.title, self.created.strftime("%Y-%m-%d %H:%M"))

    def supershort(self):
        return u"%s: %s" % (self.creator, self.created.strftime("%Y-%m-%d %H:%M"))

    def get_absolute_url(self):
        return u'/%s/?page=%d#%d' % (self.topic.id, self.get_page(), self.id)

    def save(self, *args, **kwargs):
        self.topic.user_lst = str(self.creator.id)
        self.topic.save()
        super(Post, self).save(*args, **kwargs)

    short.allow_tags = True


class ProfaneWord(models.Model):
    word = models.CharField(max_length=60)

    def __unicode__(self):
        return self.word


def send_post_email(sender, instance, **kwargs):
    if kwargs['created']:
        message = _('There is a new message on this forums you previously posted: \n\n')
        for forum in instance.topic.forums.all():
            message += '%sforum/%s%s\n\n' % (settings.HOST, forum.slug, instance.get_absolute_url())
        creators = Post.objects.filter(topic=instance.topic).values('creator__email').annotate(n=Count("creator__id"))
        for creator in creators:
            if not instance.creator.email == creator['creator__email'] and instance.creator.email_notification:
                send_mail('[' + settings.FORUM_SUBJECT + ' - ' + instance.topic.title + ']', message, settings.DEFAULT_FROM_EMAIL, [creator['creator__email']])


def send_topic_email(sender, instance, **kwargs):
    if kwargs['created']:
        message = _('New topic was created: \n\n%sadmin/django_forum_app/topic/%s') % (settings.HOST, instance.id)
        for forum in instance.forums.all():
            creator = forum.creator
            creator.email_user(subject='[' + settings.FORUM_SUBJECT + ' - ' + instance.title + ']', message=message, from_email=settings.DEFAULT_FROM_EMAIL)

post_save.connect(send_topic_email, sender=Topic)
post_save.connect(send_post_email, sender=Post)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models
from django.conf import settings


class Migration(migrations.Migration):

    dependencies = [
        migrations.swappable_dependency(settings.AUTH_USER_MODEL),
        ('photologue', '0010_auto_20160105_1307'),
    ]

    operations = [
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('order', models.IntegerField()),
                ('title', models.CharField(max_length=60)),
            ],
        ),
        migrations.CreateModel(
            name='Forum',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=60)),
                ('slug', models.SlugField(unique=True)),
                ('description', models.TextField(default=b'', blank=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('category', models.ForeignKey(to='django_forum_app.Category')),
                ('creator', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('icon', models.ForeignKey(blank=True, to='photologue.Photo', null=True)),
            ],
        ),
        migrations.CreateModel(
            name='Post',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=60, verbose_name=b'Izenburua')),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('body', models.TextField(max_length=10000)),
                ('user_ip', models.GenericIPAddressField(null=True, blank=True)),
                ('telegram_id', models.CharField(max_length=20, null=True, blank=True)),
                ('creator', models.ForeignKey(related_name='post_posts', blank=True, to=settings.AUTH_USER_MODEL, null=True)),
            ],
        ),
        migrations.CreateModel(
            name='ProfaneWord',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('word', models.CharField(max_length=60)),
            ],
        ),
        migrations.CreateModel(
            name='Topic',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('title', models.CharField(max_length=60)),
                ('description', models.TextField(max_length=10000, null=True, blank=True)),
                ('block_top', models.BooleanField(default=False)),
                ('created', models.DateTimeField(auto_now_add=True)),
                ('updated', models.DateTimeField(auto_now=True)),
                ('closed', models.BooleanField(default=False)),
                ('visits', models.IntegerField(default=0)),
                ('user_lst', models.TextField(null=True, blank=True)),
                ('creator', models.ForeignKey(blank=True, to=settings.AUTH_USER_MODEL, null=True)),
                ('forums', models.ManyToManyField(to='django_forum_app.Forum')),
            ],
        ),
        migrations.AddField(
            model_name='post',
            name='topic',
            field=models.ForeignKey(to='django_forum_app.Topic'),
        ),
    ]



from django import forms
from django_forum_app.models import Topic, Post, ProfaneWord
from tinymce.widgets import TinyMCE
from django.utils.translation import ugettext as _
from django.conf import settings

TINYMCE_BODY_CONFIG = getattr(settings, 'TINYMCE_DEFAULT_CONFIG', {})
DJANGO_FORUM_APP_FILTER_PROFANE_WORDS = getattr(settings, 'DJANGO_FORUM_APP_FILTER_PROFANE_WORDS', False)


class PostAdminForm(forms.ModelForm):
    body = forms.CharField(widget=TinyMCE(
        attrs={'cols': 80, 'rows': 30, 'placeholder': _("Your answer...")}, mce_attrs=TINYMCE_BODY_CONFIG))

    class Meta:
        model = Post
        fields = ('body',)


class TopicForm(forms.ModelForm):

    title = forms.CharField(label=_("Title"), max_length=60, required=True)
    description = forms.CharField(label=_("First message"), widget=TinyMCE(
        attrs={'cols': 80, 'rows': 30, 'placeholder': _("Write your answer here!")}, mce_attrs=TINYMCE_BODY_CONFIG))

    class Meta():
        model = Topic
        fields = ('title', 'description')


class PostForm(forms.ModelForm):
    body = forms.CharField(label=_('Body'), widget=TinyMCE(
        attrs={'cols': 80, 'rows': 30, 'class': 'my_tinymce', 'placeholder': _("Your answer...")},
        mce_attrs=TINYMCE_BODY_CONFIG,
    ))

    class Meta():
        model = Post
        exclude = ('creator', 'updated', 'created', 'topic', 'user_ip', 'telegram_id')

    def clean_body(self):
        body = self.cleaned_data["body"]

        if DJANGO_FORUM_APP_FILTER_PROFANE_WORDS:
            profane_words = ProfaneWord.objects.all()
            bad_words = [w for w in profane_words if w.word in body.lower()]

            if bad_words:
                raise forms.ValidationError(_("Bad words like '%s' are not allowed in posts.") % (reduce(lambda x, y: "%s,%s" % (x, y), bad_words)))

        return body



from django.contrib import admin
from django_forum_app.models import Category, Forum, Topic, Post, ProfaneWord
from django_forum_app.forms import PostAdminForm


class CategoryAdmin(admin.ModelAdmin):
    list_display = ["order", "title"]


class ForumAdmin(admin.ModelAdmin):
    list_display = ["title", "slug", "category", "creator", "created"]
    prepopulated_fields = {"slug": ("title",)}


class TopicAdmin(admin.ModelAdmin):
    list_display = ["title", "creator", "created", "block_top"]
    list_filter = ["creator", ]
    filter_horizontal = ('forums',)


class PostAdmin(admin.ModelAdmin):
    search_fields = ["title", "creator"]
    list_display = ["title", "topic", "creator", "created"]
    raw_id_fields = ('creator', 'topic')
    form = PostAdminForm


class ProfaneWordAdmin(admin.ModelAdmin):
    pass

admin.site.register(Category, CategoryAdmin)
admin.site.register(Forum, ForumAdmin)
admin.site.register(Topic, TopicAdmin)
admin.site.register(Post, PostAdmin)
admin.site.register(ProfaneWord, ProfaneWordAdmin)



from django.http import HttpResponseRedirect
from django.shortcuts import render, get_object_or_404
from django.core.urlresolvers import reverse
from django.template.context_processors import csrf
from django_forum_app.models import Category, Forum, Topic, Post
from django_forum_app.forms import TopicForm, PostForm
from django.contrib.auth.decorators import login_required


def index(request):
    """Main listing."""
    categories = Category.objects.all().order_by('order')
    return render(request, "django_forum_app/list.html", {'categories': categories, 'user': request.user})


def add_csrf(request, ** kwargs):
    d = dict(user=request.user, ** kwargs)
    d.update(csrf(request))
    return d


def forum(request, slug):
    """Listing of topics in a forum."""
    top_topics = Topic.objects.filter(block_top=True, forums__slug=slug).order_by("-created")
    topics = Topic.objects.filter(block_top=False, forums__slug=slug).order_by("-created")

    topics = list(top_topics) + list(topics)

    forum = get_object_or_404(Forum, slug=slug)

    return render(request, "django_forum_app/forum.html", add_csrf(request, topics=topics, forum=forum))


def topic(request, slug, topic_id):
    """Listing of posts in a topic."""
    forum = get_object_or_404(Forum, slug=slug)
    posts = Post.objects.filter(topic=topic_id).order_by("created")
    try:
        user = request.user.id
    except:
        user = None

    topic = Topic.objects.get(pk=topic_id)
    topic.sum_visits(user)
    return render(request, "django_forum_app/topic.html", add_csrf(request, forum=forum, posts=posts, pk=topic_id, topic=topic))


@login_required
def close_topic(request, slug, topic_id):
    get_object_or_404(Forum, slug=slug)
    topic = get_object_or_404(Topic, pk=topic_id)
    topic.closed = True
    topic.save()
    return HttpResponseRedirect(reverse('topic-detail', args=(slug, topic.id)))


@login_required
def post_reply(request, slug, topic_id):

    quote = request.GET.get('quote', '')
    author = request.GET.get('author', '')
    if quote:
        quote = '<blockquote>' + quote + '<footer>' + author + '</footer></blockquote>'

    forum = get_object_or_404(Forum, slug=slug)
    posts = Post.objects.filter(topic=topic_id).order_by("created").reverse()[:3]
    topic = Topic.objects.get(pk=topic_id)

    form_title = ''
    if topic.last_post():
        form_title = 'Re: ' + topic.last_post().title.replace('Re: ', '')

    default_data = {'title': form_title, 'body': 'Zure erantzuna...'}
    form = PostForm(default_data)

    if request.method == 'POST':
        quote = request.POST.get('quote', '')
        form = PostForm(request.POST)

        if form.is_valid():

            post = Post()
            post.topic = topic
            post.title = form.cleaned_data['title']
            post.body = quote + form.cleaned_data['body']
            post.creator = request.user
            post.user_ip = request.META['REMOTE_ADDR']

            post.save()
            return HttpResponseRedirect(reverse('topic-detail', args=(slug, topic.id, )))

    return render(request, 'django_forum_app/reply.html', {'form': form, 'topic': topic, 'forum': forum, 'posts': posts, 'quote': quote})


@login_required
def new_topic(request, slug):
    form = TopicForm()
    forum = get_object_or_404(Forum, slug=slug)

    if request.method == 'POST':
        form = TopicForm(request.POST)

        if form.is_valid():

            topic = Topic()
            topic.title = form.cleaned_data['title']
            topic.description = ''
            topic.creator = request.user
            topic.save()

            topic.forums.add(forum)
            topic.save()

            post = Post()
            post.title = form.cleaned_data['title']
            post.body = form.cleaned_data['description']
            post.creator = request.user
            post.user_ip = request.META['REMOTE_ADDR']
            post.topic = topic
            post.save()
            return HttpResponseRedirect(reverse('topic-detail', args=(slug, topic.id, )))

    return render(request, 'django_forum_app/new-topic.html', {'form': form, 'forum': forum})



import uuid

from setuptools import setup, find_packages
from pip.req import parse_requirements


def get_requirements(source):

    try:
        install_reqs = parse_requirements(source, session=uuid.uuid1())
    except TypeError:
        # Older version of pip.
        install_reqs = parse_requirements(source)
    required = set([str(ir.req) for ir in install_reqs])
    return required


version = '0.2'

setup(
    name='django_forum_app',
    version=version,
    description="Django forum app",
    author='Urtzi Odriozola',
    author_email='urtzi.odriozola@gmail.com',
    url='https://github.com/urtzai/django-forum-app',
    license='cc-by-sa',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    requires=['django(>=1.8)'],
    install_requires=get_requirements('requirements.txt'),
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Web Environment',
        'Framework :: Django',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Topic :: Communications',
    ],
)



from django.conf import settings

FORUM_SUBJECT = "FORUM"

POSTS_PER_PAGE = 10

DJANGO_FORUM_APP_FILTER_PROFANE_WORDS = getattr(settings, 'DJANGO_FORUM_APP_FILTER_PROFANE_WORDS', True)

TINYMCE_DEFAULT_CONFIG = {
    "language": 'eu',
    "theme": "modern",
    "height": 600,
    "plugins": [
        "advlist autolink lists link image charmap print preview anchor",
        "searchreplace visualblocks code fullscreen",
        "insertdatetime media table contextmenu paste",
    ],
    "toolbar": "styleselect | bold italic | alignleft aligncenter alignright alignjustify | bullist numlist outdent indent | link image media | code preview",
    "menubar": False,
    "media_alt_source": False,
    "media_poster": False,
    "media_dimensions": False,
    "content_css": "/static/css/tinymce_content.css",
    "valid_elements": "@[class],p[align],h1,h2,h3,h4,h5,h6,a[href|target],strong/b,div[align],br,table,tbody,thead,tr,td,ul,ol,li,img[src|alt],em",
    "extended_valid_elements": "iframe[src|name|width|height|align|frameborder|marginwidth|marginheight|scrolling],object[width|height|classid|codebase|name|id],param[name|value],embed[src|type|width|height|flashvars|wmode|bgcolor|quality|allowscriptaccess|allowfullscreen]",
}
